import pathlib
import threading,time,signal
import sys,os,types
from functools import *
from  colorama import *

init(autoreset=True)
def resource_path(relative_path):
    """ Get absolute path to resource, works for dev and for PyInstaller """
    try:
        # PyInstaller creates a temp folder and stores path in _MEIPASS
        base_path = sys._MEIPASS

    except Exception:
        base_path = os.path.abspath(".")
    p=os.path.join(base_path, relative_path)
    print(f'file path {p}')
    return p

my_open=partial(open,encoding='utf-8')
from colorama import init,Fore
init(autoreset=True)
from queue import Queue

ENCODING='utf-8'
class Job(threading.Thread):
    threads=[]
    def __init__(self,auto=True,target=None):
        threading.Thread.__init__(self,target=target if target else None)
        # The shutdown_flag is a threading.Event object that
        # indicates whether the thread should be terminated.
        self.shutdown_flag = threading.Event()
        self.target=target

        # ... Other thread setup code here ...
        if auto:
            self.start()
        Job.threads.append(self)
    def run(self):
        print('Thread #%s started' % self.ident)
 
        while not self.shutdown_flag.is_set():
            print('doing work',self.ident)
            # ... Job code here ...
            time.sleep(1)
 
        # ... Clean shutdown code here ...
        print('Thread #%s stopped' % self.ident)

    def stop(self):
        self.shutdown_flag.set()
       

    @staticmethod
    def stop_all():
        for t in Job.threads:
            t.stop()
        for t in Job.threads:
            t.join(1)
        Job.threads=[]
class ServiceExit(Exception):
    """
    Custom exception which is used to trigger the clean exit
    of all running threads and the main program.
    """
    pass
 
 
def service_shutdown(signum, frame):
    print('Caught signal number  %d' % signum)
    raise ServiceExit

# signal.signal(signal.SIGTERM, service_shutdown)
# signal.signal(signal.SIGINT, service_shutdown)





container_list=Queue()

class SavePrinter:
    def __init__(self) -> None:
        self.lock=threading.Lock()
        pass

    def __call__(self, *args,**kwds) :
        txts=[str(e) for e in args]
        txt=''.join(txts)
        container=kwds.get('container')
        all_=kwds.get('msg')
        with self.lock:
            print(txt,flush=True)

        if not container:
            container_list.put(txt)
        else:
            if all_:
                container.put(txt)
            else:
                for t in txt:
                    if not t.isascii():
                        pass
                    else:
                       return
                container.put(txt)




save_print=SavePrinter()
import shlex
from subprocess import Popen, PIPE

def get_exitcode_stdout_stderr(cmd):
    """
    Execute the external command and get its exitcode, stdout and stderr.
    """
    args = shlex.split(cmd)

    proc = Popen(args, stdout=PIPE, stderr=PIPE)
    out, err = proc.communicate()
    exitcode = proc.returncode
    #
    return exitcode, out, err

# cmd = "..."  # arbitrary external command, e.g. "python mytest.py"
# exitcode, out, err = get_exitcode_stdout_stderr(cmd)



# main()
from typing import Any
from random import choice
from glob import glob
colors=[
    Fore.GREEN,Fore.LIGHTYELLOW_EX,Fore.CYAN,Fore.LIGHTBLUE_EX,Fore.YELLOW,Fore.LIGHTGREEN_EX,Fore.LIGHTBLUE_EX
]
def get_input(prompt, default: Any = '', block: Any = True, validators=None, fa_msg='',file=False,base_path=None):
    if validators is None:
        validators = []
    color=choice(colors)
    while True:
        try:
            v = input(color + '[-] ' + prompt.upper() +f' {"blocking " if block else  ""}' + str(
                f'[ defualt {default} ]' if default or default==0 else '[do-default-value]') +  ': '.upper())
            if not v:
                if default or default==0:
                    v = default
                else:
                    if not block:
                        return v
                    else:
                        continue
            else:
                pass
            passed = 0
            print(f'Your Input {v}')
            if validators and v:
                # print('checking your input', str(v))
                for validator in validators:
                    try:
                        if validator(str(v)):
                            passed += 1
                    except Exception:
                        continue
                if passed == len(validators):
                    pass
                else:
                    print('Failed input ' + (f"must be {fa_msg}" if fa_msg else ''))
                    continue
            if v and '.txt' in str(v) or file:
                try:
                    if base_path:
                        file_path=os.path.join(base_path,v)
                    else:
                        print('no base path')
                        file_path=v
                    print(f'File path {file_path}')
                    v=file_path
                    open(file_path)
                except FileNotFoundError:
                    print('File not Found')
                    continue
            # print(f'Your Input {v}')
            return v
        except KeyboardInterrupt:
            sys.exit('bye')

        except Exception as e:
            print('Error',e)
            continue


def get_as_bytes(text):
    text=bytes(text.strip(),'utf-8')
    return text
def start_new_thread(func,*args,**kwargs):
    def inner():
            func(*args,**kwargs)

    t=threading.Thread(target=inner)
    t.start()

    return t
old_open=open
open=partial(open,encoding='utf-8')
get_file_input=partial(get_input,file=True)



from colorama import Fore,init
init(autoreset=True)

def red(*args):
    text = ''
    for arg in args:
        text +=str(arg)+ ' '
    # print(text)
    text=Fore.RED+text
    return text



def yellow(*args):
    text = ''
    for arg in args:
        text += Fore.YELLOW+str(arg)+ ' '
    # print(text)
    text = Fore.YELLOW + text
    return text


def green(*args):
    text = ''
    for arg in args:
        text += str(arg) +' '
    # print(text)
    text = Fore.GREEN + text
    return text


def cyan(*args):
    text = ''
    for arg in args:
        text += str(arg)+ ' '
    # print(text,end='')
    text = Fore.CYAN + text
    return text

def info(*args):
    text = 'Info: '
    for arg in args:
        text += Fore.CYAN+str(arg) +' '
    # print(text)
    text = Fore.CYAN + text
    return text


lock=threading.RLock()

def debug(*args,**kwargs):
    color=kwargs.get('color')
    with lock:
        txt=' '.join([str(a) for a in args])
        txt=f'[{threading.current_thread().name}]--'+txt
        if color:
            print(color+txt)
        else:
            print(txt)
        time.sleep(.2)

def red_debug(*args):
    debug(*args,color=Fore.RED)

def yellow_debug(*args):
    debug(*args,color=Fore.YELLOW)

def cyan_debug(*args):
    debug(*args, color=Fore.CYAN)
def green_debug(*args):
    debug(*args, color=Fore.GREEN)


def magneta_debug(*args):
    debug(*args,color=Fore.MAGENTA)

def blue_debug(*args):
    debug(*args,color=Fore.BLUE)


class Mylocal(threading.local):
    def __init__(self):
        super(Mylocal, self).__init__()
        self.color=''
        self.accounts=[]
        self.counter=0


def create_files(files,folder='',base_path=''):
    # save_path=f'/{folder}'
    new_files=[]
    if folder:
        print(f'folder {folder}')

        folder=os.path.join(base_path,folder)
        if not os.path.exists(folder):
            os.makedirs(folder)
    if isinstance(files,str):
        files=[files]
    if isinstance(files,list):
        for file in files:
            # print(file)
            assert isinstance(file,str)
            if not file.endswith('.txt'):
                file=file+'.txt'
            if folder:
                file_path=pathlib.Path(folder).joinpath(f'{file}')
            else:
                file_path=pathlib.Path(file)
            file_path.touch(exist_ok=True)
            new_files.append(file_path)
        return new_files
    else:
        if folder:
            file_path = pathlib.Path(folder).joinpath(f'{file}')
        else:
            file_path = pathlib.Path(file)
        f = pathlib.Path(file_path)
        f.touch(exist_ok=True)
        return f


local_storage=Mylocal()
not_load_event=threading.Event()



def en_int(x):
	return int(x)


def en_contain(*args):

	def check(x):
		# print(x)
		argss=list(args)
		for a in argss:
			# print(a ,x)
			if str(a) in x:
				return True
			else:
				pass
		# vv=[str(v) for v in args]
		# print(vv)
		# return x in vv
	return check


# a=en_contain(':')
# a=a('dfs:sdfsdf',)
#
# print(a)


# The “abstractions” that callers will see.

class Logger(object):
    def __init__(self, handler):
        self.handler = handler

    def log(self, message):
        self.handler.emit(message)

class FilteredLogger(Logger):
    def __init__(self, pattern, handler):
        self.pattern = pattern
        super().__init__(handler)

    def log(self, message):
        if self.pattern in message:
            super().log(message)

# The “implementations” hidden behind the scenes.

class FileHandler:
    def __init__(self, file):
        self.file = file

    def emit(self, message):
        self.file.write(message + '\n')
        self.file.flush()

class SocketHandler:
    def __init__(self, sock):
        self.sock = sock

    def emit(self, message):
        self.sock.sendall((message + '\n').encode('ascii'))

class SyslogHandler:
    def __init__(self, priority):
        self.priority = priority

    def emit(self, message):
        syslog.syslog(self.priority, message)


import time
import threading
import warnings
from queue import Empty
from pathlib import *
from urllib3 import disable_warnings
from .Extensions import  *
disable_warnings()
from threading import Thread,Event,Lock
from os import system as terminal
import requests
import socket
import traceback
import requests
import os
import sys
import bs4
from bs4 import BeautifulSoup
import random
from threading import active_count
from requests.exceptions import *
from glob import glob
from .file1 import *
from .validtors import *

class Local(threading.local):
    def __init__(self):
        super(Local, self).__init__()
        self.color=''
        self.counter=0


local_storage=Local()



# uncompyle6 version 3.5.0
# Python bytecode 2.7 (62211)
import sys,os
sys.path.append(os.getcwd())
from threading import Lock
import requests, uuid, time, os
from colorama import init
from termcolor import colored
import urllib3, re, oauth2, json, tweepy, string, sys
from random import choice, randint
import urllib.parse
import threading
from time import sleep

lock=threading.RLock()

# quote_plus=urllib.parse.quote_plus

init()


def random_char(y=8):
    return ('').join(choice(string.ascii_letters) for x in range(y))




def red(*args):
    text = ''
    for arg in args:
        text += colored(arg, 'red') + ' '

    return text


def yellow(*args):
    text = ''
    for arg in args:
        text += colored(arg, 'yellow') + ' '

    return text


def green(*args):
    text = ''
    for arg in args:
        text += colored(arg, 'green') + ' '

    return text


def cyan(*args):
    text = ''
    for arg in args:
        text += colored(arg, 'cyan') + ' '

    return text


def magenta(*args):
    text = ''
    for arg in args:
        text += colored(arg, 'magenta') + ' '

    return text


must_re_settings = False
save_settings = 'y'
loadsettings = ''
def get_setting(json_object=None, setting_name='', prompt='', default=None, block=False,globals={}):
    global save_settings
    var_name = setting_name
    if 'file' in setting_name:
        file = True
    else:
        file = False
    setting_name = setting_name.replace('_', '-')
    if json_object is None:
        json_object = json_object
    if loadsettings == 'n' or must_re_settings:
        globals[var_name] = get_input(prompt or setting_name, file=file, default=default,
                                        block=block if block else False)
        return globals[var_name]
    try:
        varibal = json_object[setting_name.upper()]
    except Exception as e:
        print(f'setting {setting_name.upper()} not found Enter It')
        varibal = get_input(prompt or setting_name, default=default, file=file, block=block if block else False)
        save_settings = 'y'
    return varibal


def load(settings_container=[],globals={},file='settings.json',base_path=None):
    if not base_path:
        warnings.warn('You Should Enter PAth')
        sys.exit('bye')
    if base_path:
        settings_file=Path(base_path).joinpath(file)
    else:
        settings_file=Path(file)
    settings_file.touch(exist_ok=True)
    global must_re_settings, loadsettings, save_settings
    print('must_re-settings', must_re_settings)
    json_obj = {}
    try:
        with open(settings_file, 'r') as f:
            if not f.read():
                print('Empty File')
            f.seek(0)
            json_obj = json.load(f)
            debug('#' * 30)
            debug(Fore.GREEN + f'prev settings.......')
            for key_, val in json_obj.items():
                debug(f'{key_.upper()}=={val}')
            debug('#' * 30)
            if not json_obj:
                raise ValueError('No Settings')
    except json.JSONDecodeError:
        with settings_file.open('w') as setting_f :
            data={}
            json.dump(data, setting_f, ensure_ascii=False, indent=4)
        return load()
    except ValueError as e :
        print(f'Erorr {e}')
        must_re_settings = True
    except Exception as e:
        debug(Fore.RED + f'No Prevoius Settings Detected  choose [n]: \n Error--{e.__class__}')
        traceback.print_exc()
        must_re_settings = True
        pass
    if not must_re_settings:
        loadsettings = input(Fore.BLUE + '[--]Want to load prev settings [y/n] :  ') or 'y'
    else:
        loadsettings = 'n'
    try:
        for setting_name, default_val, blocking, type_ in settings_container:
            globals[setting_name] = type_(
                get_setting(json_object=json_obj, setting_name=setting_name, default=default_val, block=blocking,globals=globals))
            print(f'{setting_name} Now {globals[setting_name]}')
            print('*' * 50)

    except Exception as e:
        debug('Error load again', e)
        traceback.print_exc()
        load()
    save_settings = input(' [--] save settings [y/n]') or 'y'
    if loadsettings == 'n' or must_re_settings:
        pass
    if save_settings == 'y':
        with open(settings_file, 'w', encoding='utf-8') as setting:
            data = {}
            for setting_name, default_val, blocking, type_ in settings_container:
                data[setting_name.replace('_', '-').upper()] = type_(globals[setting_name])
            json.dump(data, setting, ensure_ascii=False, indent=4)
            
            

def create_and_open(file,ext='.txt',base_path=None):
    if not file.endswith(ext):
        file+=ext
    if base_path:
        file=Path(base_path).joinpath(file)

    # print(f'file {file.as_posix()}')
    file=Path(file)
    file.touch(exist_ok=True)
    return file
    


import pathlib
from setuptools import setup,find_packages

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="main-importsss",
    version="1.0.0",
    description="Read ",
    long_description=README,
    author="Fahmy",
    author_email="Fahmy@realpython.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    packages=find_packages(),
    include_package_data=True,
    install_requires=[],
    
)


import requests
import functools
import html,string
from .file1 import debug
def make_soap( res: requests.Response):
    return BeautifulSoup(html.unescape(res.text), 'html.parser')




def password_generator(size=8, chars=string.digits, word=False, addon=''):
    n = ''.join(random.choice(chars) for _ in range(size))
    if addon:
        n = addon + n
    if word:
        # debug(Back.BLUE+'word genrated', n)
        return n
    return n


def bad_key(method):
    @functools.wraps(method)
    def wrapper(*args,**kwargs):
        try:
            return method(*args,**kwargs)
        except ErrorsModel as e:
            if 'BAD_KEY' in e.args:
                debug('wrong api key')
                raise SystemExit("exiting")
    return wrapper
def NOCONNECTON_ERROR_SPECKER(fn):
    @functools.wraps(fn)
    @speaker
    def wrapper(*args,**kwargs):
        while True:
            try:
                return fn(*args,**kwargs)
            except requests.RequestException as e:
                debug('CONNECTION ERROR Retrying',e)

                # time.sleep(2)
                pass
    return wrapper




import sys

import requests.exceptions

import threading,time
import traceback
from functools import wraps
from colorama import Fore,init
init(autoreset=True)


class RequestException(Exception):
    pass
def no_connection_error(f):
    name=f.__qualname__
    @logger
    @wraps(f)
    def wrapper(*args,**kwargs):
        tries = 0
        while True:
            tries+=1
            if tries >5:
                break
            try:
                r=f(*args,**kwargs)
                return r
            # except requests
            except requests.exceptions.RequestException as e :
                debug(f"request error  {name}",e,f'Retry{tries}')
                # time.sleep(1)
            except RequestException as e:
                debug(f'Sim Request error {name} {f}',e)
                time.sleep(3)
            except TimeoutError:
                debug('opertaion timed out')
                return
            except Exception as e:
                if 'TRY_AGAIN_LATER' in e.args:
                    debug(f'sleeping for 20 seconds and retry again...... {name}')
                    time.sleep(20)
                elif 'ERROR_NO_OPERATIONS' in e.args:
                    return
                elif 'BANNED' in  e.args:
                    debug('User Banned Sleep 60 Seconds')
                    time.sleep(60)
                elif 'EXCEEDED_CONCURRENT_OPERATIONS' in e.args:
                    debug('many mnubers')
                    time.sleep(60)
                    return
                elif 'NO_COMPLETE_TZID' in e.args:
                    debug('NO_COMPLETE_TZID')
                    time.sleep(20)
                    return
                elif 'ERROR_WRONG_TZID' in e.args:
                    return
                elif 'WARNING_LOW_BALANCE' in e.args:
                    debug(Fore.YELLOW+'No balance')
                    sys.exit('bye')
                elif 'NO_NUMBER' in e.args:
                    debug('NO_NUMBER')
                    time.sleep(1)
                elif 'NO_BALANCE' in e.args:
                    debug('NO_BALANCE'.upper())
                    # time.sleep(4)
                    sys.exit(debug('exit thread '))
                else:
                    debug(f'unknown error on {f}   ', e)
                    traceback.print_exc()
    return wrapper
def logger(func):
    name=func.__qualname__
    @speaker
    @wraps(func)
    def wrapper(*args,**kwargs):
            result=func(*args,**kwargs)
            if result:
                # val=type(result)
                if isinstance(result,(str,int)):
                    val=result
                else:
                    val=type(result)

            else:
                val='NoneVAl'
            debug(f'{name} return {val}')
            return result
    return wrapper


def speaker(func):
    @wraps(func)
    def wrapper(*args,**kwargs):
        debug(Fore.CYAN+f'-Calling Function ',func.__qualname__)
        while True:
            try:
                 r=func(*args,**kwargs)
                 return r
            except requests.exceptions.RequestException as e:
                debug('connection error',e)
                time.sleep(1)
                continue
                # return func(*args,**kwargs)
    return wrapper

# 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